<?php

require_once("DataObject.class.php");
require_once("Authority.class.php");

class User extends DataObject {
	
	
	const PERM_MODIFICATION = "RW";
	
	
  protected $objectName = "users";
  protected $prettyName = "Utilisateur";

  protected $email;
  protected $subject_dn;
  protected $issuer_dn;
  protected $name;
  protected $givenname;
  protected $telephone;
  protected $role;
  protected $authority_group_id;
  protected $authority_id;
  protected $status;
  protected $certificate;
  protected $cert_not_before;
  protected $cert_not_after;
  protected $cert_serial;
	protected $certificate_hash;
  protected $certFilePath;
  protected $certPassphrase;
  protected $dbFields = array( "email" => array( "descr" => "Adresse lectronique", "type" => "isEmail", "mandatory" => true),
						 "subject_dn" => array( "descr" => "Dn du certificat", "type" => "isString", "mandatory" => true),
						 "issuer_dn" => array( "descr" => "DN du fournisseur du certificat", "type" => "isString", "mandatory" => true),
						 "name" => array( "descr" => "Nom", "type" => "isString", "mandatory" => true),
						 "givenname" => array( "descr" => "Prnom", "type" => "isString", "mandatory" => true),
 						 "role" => array( "descr" => "Rle", "type" => "isString", "mandatory" => true),
 						 "telephone" => array( "descr" => "Tlphone", "type" => "isString", "mandatory" => false),
						 "authority_group_id" => array( "descr" =>  "Groupe", "type" => "isInt", "mandatory" => false),
						 "authority_id" => array( "descr" =>  "Collectivit", "type" => "isInt", "mandatory" => true),
						 "status" => array( "descr" => "tat", "type" => "isInt", "mandatory" => true),
						 "certificate" => array( "descr" => "Certificat de l'utilisateur", "type" => "isString", "mandatory" => false),
						 "cert_not_before" => array( "descr" => "Date d'mission du certificat", "type" => "isDate", "mandatory" => true),
						 "cert_not_after" => array( "descr" => "Date d'expiration du certificat", "type" => "isDate", "mandatory" => true),
						 "cert_serial" => array( "descr" => "Numro de srie du certificat", "type" => "isDate", "mandatory" => true),
  						"login" => array("descr" => "login","type"=>"isString","mandatory"=>false),
  						"password" => array("descr" => "password","type"=>"isString","mandatory"=>false),
  						"certificate_rgs_2_etoiles" => array("descr" => "Certificat RGS**","type"=>"isString","mandatory"=>false),
	  "certificate_hash" => array("descr", "Certificat fingerprint", "type" => "isString", "mandatory" => false),

						 );
  protected $roleTypes = array(
							   "SADM" => "Super administrateur",
							   "GADM" => "Administrateur de groupe",
							   "ADM" => "Administrateur collectivit",
							   "USER" => "Utilisateur"
							   );
  protected $permsTypes = array(
								"NONE" => "Aucune",
								"RO" => "Visualisation",
								"RW" => "Modification",
								);
  protected $superPermsTypes = array(
									 "GRANT" => "Concession"
									 );
  private $perms;
  private $is_loggued;
  
  /**
   * \brief Constructeur d'un utilisateur
   * \param $id integer (optionnel) : numro d'identifiant de l'utilisateur
  */
  public function __construct($id = false) {
    parent::__construct($id);
  }

  /**
   * \brief Mthode de vrification de la prsence d'utilisateur dans la base
   * \return True si la base contient au moins un utilisateur, false sinon
  */
  public static function dbHasUser() {
	$sql = "SELECT id FROM users";

    $db =DatabasePool::getInstance();

	$result = $db->select($sql);

    if (! $result->isError()) {
		  if ($result->num_row() > 0) {
			return true;
		  } else {
			return false;
		  }
    } else {
		  echo $result->error;
		  return false;
		}
  }

	public function getNbUserWithMyCertificate(){
		$sql = "SELECT count(*) AS nb FROM users WHERE certificate_hash='" .
			pg_escape_string($this->certificate_hash) .
			"'";

		$result = $this->db->select($sql);
		$row = $result->get_next_row();
		return  $row['nb'];
	}

  /**
   * \brief Mthode d'authentification de l'utilisateur
   * \return true si succs, false sinon
   *
   * Cette mthode vrifie qu'un utilisateur est bien autoris  se connecter au systme.
   * Elle se base sur les donnes du certificat prsent au serveur Web pour authentifier
   * et initialiser les donnes de l'utilisateur.
  */
	public function authenticate() {
		$authenfication = Authentification::getInstance();
		$this->id = $authenfication->authenticate();

	  	$this->is_loggued = true;
	  	$init = $this->init();
	  	$is_active = $this->isActive();
	  	return $init && $is_active;
	}
	
  /**
   * \brief Mthode d'initialisation d'un utilisateur depuis la base de donnes
   * \return true si succs, false sinon
  */
  public function init() {
	return (parent::init() && $this->initPerms());
  }
 
  /**
   * \brief Mthode d'initialisation des permissions d'un utilisateur depuis la base de donnes
   * \return true si succs, false sinon
  */
  public function initPerms() {
	if (isset($this->id)) {
	  $this->resetPerms();

	  $authModules = Module::getModulesForAuthority($this->authority_id);

	  $sql = "SELECT users_perms.id, users_perms.module_id, users_perms.perm, modules.name " .
	  		" FROM users_perms LEFT JOIN modules ON users_perms.module_id=modules.id " .
	  		" WHERE users_perms.user_id='" . $this->id . "' AND modules.status=1";

	  $result = $this->db->select($sql);

	  if (! $result->isError()) {
		while ($row = $result->get_next_row()) {
		  // Ajout de la permission uniquement si la collectivit est autorise sur ce module
		  if ($this->isGroupAdminOrSuper() || ! empty($authModules[$row["module_id"]])) {
			$this->perms[$row["name"]] = array("module_id" => $row["module_id"], "perm" => $row["perm"], "id" => $row["id"]);
		  }
		}
	  } else {
		$this->errorMsg = "User::initPerms - erreur de rcupration des permissions de l'utilisateur";
		return false;
	  }

	}

	return true;
  }
	
  /**
   * \brief Mthode de remise  zro des permission de l'objet utilisateur courant
  */
  public function resetPerms() {
	$this->perms = array();
  }
	
  /**
   * \brief Mthode qui dtermine si l'utilisateur est un administrateur de groupe ou un super administrateur
   * \return true si l'utilisateur est administrateur de groupe ou super administrateur, false sinon
  */
  public function isGroupAdminOrSuper() {
	return ($this->isGroupAdmin() || $this->isSuper());
  }
  
  /**
   * \brief Mthode qui dtermine si l'utilisateur est un administrateur de groupe
   * \return true si l'utilisateur est administrateur de groupe, false sinon
  */
  public function isGroupAdmin() {
	if (isset($this->role) && ($this->role == "GADM") && is_numeric($this->authority_group_id)) {
	  return true;
	} else {
	  return false;
	}
  }
	
  /**
   * \brief Mthode qui dtermine si l'utilisateur est un super administrateur
   * \return true si l'utilisateur est super administrateur, false sinon
  */
  public function isSuper() {
    if ( isset($this->role) && $this->role == 'SADM') {
	  return true;
	} else {
	  return false;
	}
  }
	
  /**
   * \brief Mthode qui dtermine si l'utilisateur est activ ou non
   * \return true si l'utilisateur est activ, false s'il est dsactiv
  */
  public function isActive() {
	$authority = new Authority($this->authority_id);

	// Prise en compte du groupe
	if (is_numeric($authority->get("authority_group_id"))) {
	  $group = new Group($authority->get("authority_group_id"));
	  $groupIsActive = $group->isActive();
	} else {
	  // La collectivit n'appartient  aucun groupe
	  $groupIsActive = true;
	}

    return ($this->status == 1 && $authority->isActive() && $groupIsActive);
  }
	
	public function isLogged(){
		return $this->is_loggued;
	}

	public function login($login,$password){
		$this->retrieveInfoFromClientCertificate();

		$sql = "SELECT id FROM users WHERE certificate_hash='" . pg_escape_string($this->certificate_hash) . "'" .
                        " AND login='".pg_escape_string($login)."' AND password='".pg_escape_string($password)."'";

	 	$result = $this->db->select($sql);
		if ($result->isError() || $result->num_row() != 1){
			$this->errorMsg = "User::getIdFromCertData - chec du mappage de l'utilisateur depuis les informations du certificat";
			return false;
		}

   		$row = $result->get_next_row();
		$this->id = $row['id'];
		$_SESSION['id_login'] = $this->id;
		return true;
	}

	private function retrieveInfoFromClientCertificate(){
		 // Ne marche pas avec apache-ssl
		if ( ! isset($_SERVER['SSL_CLIENT_VERIFY']) || $_SERVER['SSL_CLIENT_VERIFY'] != "SUCCESS") {
			return false;
		}

		if (empty($_SERVER['SSL_CLIENT_CERT'])){
			return false;
		}

		if (($tab = openssl_x509_parse($_SERVER['SSL_CLIENT_CERT'])) === false) {
	       	return false;
		}

		// Si l'utilisateur est authentifi par certificat
	    $this->issuer_dn = "";
        foreach ($tab['issuer'] as $key => $val) {
        	$this->issuer_dn .= "/" . $key . "=" . utf8_decode($val);
		}

		// Si l'utilisateur est authentifi par certificat
		$this->subject_dn = "";
		foreach ($tab['subject'] as $key => $val) {
			$this->subject_dn .= "/" . $key . "=" . utf8_decode($val);
		}

		$x509Certificate = new X509Certificate();
		$this->certificate_hash = $x509Certificate->getBase64Hash($_SERVER['SSL_CLIENT_CERT'], UserSQL::CERTIFICATE_FINGERPRINT_HASH_ALG);

	}

	public function logout(){
		unset($_SESSION['id_login']);
	}

	public function getCertificateInfo(){
		$this->retrieveInfoFromClientCertificate();
		return array('subject' => $this->subject_dn, 'issuer' => $this->issuer_dn,'certificate_hash' => $this->certificate_hash);
	}

  /**
   * \brief Mthode de rinitialisation de la session d'un utilisateur
  */
  public function resetSession() {
    $_SESSION = array();

    $sessionCookie = session_get_cookie_params();

    if ( (empty($sessionCookie['domain'])) && (empty($sessionCookie['secure'])) ) {
      setcookie(session_name(), '', time()-3600, $sessionCookie['path']);
    } elseif (empty($sessionCookie['secure'])) {
      setcookie(session_name(), '', time()-3600, $sessionCookie['path'], $sessionCookie['domain']);
    } else {
      setcookie(session_name(), '', time()-3600, $sessionCookie['path'], $sessionCookie['domain'], $sessionCookie['secure']);
    }

    session_destroy();
  }

  /**
   * \brief Mthode qui dtermine si l'utilisateur est un administrateur, un administrateur de groupe ou un super administrateur
   * \return true si l'utilisateur est administrateur, administrateur de groupe ou super administrateur, false sinon
  */
  public function isAdmin() {
	if (isset($this->role) && ($this->role == "SADM" || $this->role == "GADM" || $this->role == "ADM")) {
	  return true;
	} else {
	  return false;
	}
  }

  /**
   * \brief Mthode qui dtermine si l'utilisateur est un administrateur de collectivit
   * \return true si l'utilisateur est administrateur de collectivit, false sinon
  */
  public function isAuthorityAdmin() {
	if (isset($this->role) && $this->role == "ADM") {
	  return true;
	} else {
	  return false;
	}
  }

  /**
   * \brief Mthode qui dtermine si l'utilisateur courant  les droits pour modifier un autre utilisateur
   * \param $id integer : Numro d'identifiant de l'utilisateur a diter
   * \return true si l'utilisateur peut modifier, false sinon
  */
  public function canEditUser($id) {
	$sql = "SELECT authority_id FROM users WHERE id='" . $id . "'";

    $result = $this->db->select($sql);

    if (! $result->isError() && $result->num_row() == 1) {
      $row = $result->get_next_row();

	  $authority = new Authority($row["authority_id"]);

	  // Prise en compte du groupe
	  if (is_numeric($authority->get("authority_group_id"))) {
		$group = new Group($authority->get("authority_group_id"));
		$inGroup = $this->role == "GADM" && $this->authority_group_id == $group->getId();
	  } else {
		// La collectivit n'appartient  aucun groupe
		$inGroup = false;
	  }

      return ($this->role == "SADM" || $inGroup || ($row["authority_id"] == $this->authority_id && $this->role == "ADM"));
    } else {
      $this->errorMsg = "User::canEditUser - erreur de rsultat requte base de donnes";
      return false;
    }
  }

  /**
   * \brief Mthode qui dtermine si l'utilisateur courant  les droits pour concder des droits sur un module
   * \param $name chaine : Nom du module
   * \return true si l'utilisateur peut concder, false sinon
  */
  public function canGrantModule($name) {
	if (strcmp($this->getPerm($name), "GRANT") == 0) {
	  return true;
	} else {
	  return false;
	}
  }

  /**
   * \brief Mthode retournant les permissions de l'utilisateur sur un module particulier
   * \param $module chane : nom du module pour lequel rcuprer les permissions
   * \return La chane des permissions sur le module ou null si aucune permission trouve
  */
  public function getPerm($module) {
  	if (empty($this->perms[$module])){
  		return null;
  	}
  	return $this->perms[$module]["perm"];
  }

  /**
   * \brief Mthode retournant la description du rle de l'utilisateur en cours
   * \return La description ou une chane vide si la decsription n'est pas trouve
  */
  public function getRoleDescr() {
    return (isset($this->role)) ? $this->roleTypes[$this->role] : "";
  }

  /**
   * \brief Mthode retournant les permissions de l'utilisateur en cours sur les modules
   * \return Un tableau de permissions ou null si les permissions ne sont pas dfinies
  */
  public function getPerms() {
    return (isset($this->perms)) ? $this->perms : null;
  }

  /**
   * \brief Mthode dterminant si un utilisateur peut accder  une page
   * \param $module chane : nom du module
   * \return True si l'utilisateur peut accder ou false sinon
  */
	public function canAccess($module) {
		if ($this->isGroupAdminOrSuper()) {
  			return true;
  		} 	
	  	$authority = new Authority($this->authority_id);

	  	$mod = new Module();
	  	if (! $mod->initByName($module)) {
			return false;
	  	}

	  	if (! $authority->getModulePerm($mod->getId())) {
			return false;
	  	}
	  	 	
	  	if ($this->getPerm($module) != "NONE" && $this->getPerm($module)!= null) {
	 		return true;
	  	}
	  	return false;
  }

  public function checkDroit($module, $droit){
  	if ($this->isGroupAdminOrSuper()) {
  		return true;
  	}
  	$authority = new Authority($this->authority_id);

  	$mod = new Module();
  	if (! $mod->initByName($module)) {
  		return false;
  	}
  	
  	if (! $authority->getModulePerm($mod->getId())) {
  		return false;
  	}

  	if ($this->getPerm($module) == "RW" ) {
  		return true;
  	}

  	if ($this->getPerm($module) == $droit){
  		return true;
  	}
  	return false;
  }
  
  /**
   * \brief Mthode dterminant si un utilisateur  accs en modification
   * \param $module chane : nom du module
   * \return True si l'utilisateur peut modifier ou false sinon
  */
	public function canEdit($module) {
		if ($this->isGroupAdminOrSuper()) {
			return true;
		}
		$authority = new Authority($this->authority_id);

		$mod = new Module();
		if (! $mod->initByName($module)) {
			return false;
		}

		if (! $authority->getModulePerm($mod->getId())) {
			return false;
		}
		if ($this->getPerm($module) == "RW") {
			return true;
		}
		return false;
  }

  /**
   * \brief Mthode permettant de fixer les permissions d'un utilisateur sur un module
   * \param $module_id integer : numro d'identifiant du module dont fixer les permissions
   * \param $perm chane : permission sur le module
   * \return true si succs, false sinon
  */
  public function setPerm($module_id, $perm,array $specific_perms = array()) {
	$module = new Module($module_id);
	if (! $module->init()) {
	  return false;
	}

	$all_perms = $this->getPermTypes($specific_perms);

	  if ( empty($all_perms[$perm])){
		$perm = "NONE";
	}

	$this->perms[$module->get("name")] = array("module_id" => $module->getId(), "perm" => $perm);
	return true;
  }

	public function getPermTypes(array $module_specific_perms = array()){
		$result = $this->permsTypes;

		if ($this->isSuper()) {
			$result = array_merge($result, $this->superPermsTypes);
		}

		$result = array_merge($result,$module_specific_perms);

		return $result;
	}

  /**
   * \brief Mthode renvoyant le nom d'un utilisateur formatt "Prnom Nom"
   * \return La chane du nom de l'utilisateur
  */
  public function getPrettyName() {
    if (strlen($this->name) > 0) {
      $str = $this->givenname .  " " . $this->name;
    } else {
      $str = $this->login;
    }

    return $str;
  }

  /**
   * \brief Mthode d'enregistrement d'un utilisateur dans la base de donnes
   * \param $validate boolen (optionnel) Demande la validation ou non des donnes de l'entit avant enregistrement (dfaut : true)
   * \return true si succs, false sinon
  */
  public function save($validate = true,$bouchon_4_strict_standard = true) {
	if (isset($this->certFilePath)) {
	  if (! $this->importCert()) {
		return false;
	  }
	}

    if (! ($sql = parent::save($validate, true))) {
	  return false;
	}

	if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

    if (! $this->db->exec($sql)) {
      $this->errorMsg = "Erreur lors de la sauvegarde de l'utilisateur.";
	  $this->db->rollback();
      return false;
    }

	// Traitement permissions sur les modules
	$sql = "DELETE FROM users_perms WHERE user_id=" . $this->id;

    if (! $this->db->exec($sql)) {
      $this->errorMsg = "Erreur lors de la rinitialisation des permissions de l'utilisateur.";
	  $this->db->rollback();
      return false;
    }

	if (isset($this->perms)) {
	  reset($this->perms);
	  foreach ($this->perms as $perm) {
		$sql = "INSERT INTO users_perms (module_id, user_id, perm) VALUES(" . addslashes($perm["module_id"]) . ", " . $this->id . ", '" . addslashes($perm["perm"]) . "')";

		if (! $this->db->exec($sql)) {
		  $this->errorMsg = "Erreur lors de la sauvegarde des permissions de l'utilisateur.";
		  $this->db->rollback();
		  return false;
		}
	  }
	}

	if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

    return true;
  }

  /**
   * \brief Mthode d'import des informations contenus dans le certificat utilisateur
   * \return true si succs, false sinon
  */
  private function importCert() {
	if (isset($this->certFilePath)) {
      if (! $this->certificate = file_get_contents($this->certFilePath)) {
		$this->errorMsg = "Erreur de traitement du certificat.";
		return false;
	  }

      if (($tab = openssl_x509_parse($this->certificate)) === false) {
		$this->errorMsg = "Erreur d'analyse du certificat.";
		return false;
	  }

	  $this->issuer_dn = "";
	  foreach ($tab['issuer'] as $key => $val) {
		$this->issuer_dn .= "/" . $key . "=" . utf8_decode($val);
	  }

      $this->subject_dn = $tab["name"];

		/*
			Il semble qu'il y a un bug avec l'utilisation des validFrom et validTo qui retourne
		une date au format GeneralizedTime ou UTCTime. Les champs validFrom_time_t et
		validTo_time_t semble plus *stable*.
		*/
      $this->cert_not_before = date("Y-m-d H:i:s",$tab['validFrom_time_t']);
      $this->cert_not_after = date("Y-m-d H:i:s",$tab['validTo_time_t']);


      $this->cert_serial = $tab["serialNumber"];


		$x509 = new X509Certificate();
		$this->certificate_hash = $x509->getBase64Hash($this->certificate, UserSQL::CERTIFICATE_FINGERPRINT_HASH_ALG);


	  // Controle de l'existence d'un utilisateur avec les mmes donnes de certificat.
	  // Si un utilisateur a les mmes donnes mais qu'il s'agit de l'utilisateur courant
	  // on accepte => permet de modifier le certificat
		$ids = $this->getIdFromCertData($this->certificate_hash);

	  if ((! empty($ids) && $this->isNew()) || (! empty($ids) && ! $this->isNew() && $ids[0] != $this->id)) {

	  	if ($ids[0]){
	  		$autre = new User($ids[0]);
	  		$autre->init();
	  		if (! $autre->get('login')){
	  			$this->errorMsg = "Un utilisateur avec les mmes donnes de certificat existe dj. Vous pouvez mettre un login/mot de passe pour les diffrencier";
  				return false;
	  		}
	  	}

	  	if ($this->login) {
	  		$id = $this->getIdFromLogin($this->login);
  			if ($id){
  				$this->errorMsg = "Un utilisateur avec le mme login existe dj.";
  				return false;
  			}

	  		return true;
	  	}
		$this->errorMsg = "Un utilisateur avec les mmes donnes de certificat existe dj. Vous pouvez mettre un login/mot de passe pour les diffrencier";
		return false;
	  }

	  return true;
	}

	$this->errorMsg = "Pas de certificat fournit.";
	return false;
  }

	public function getIdFromCertData($certificate_hash)
	{
		$resultat = array();
		$sql = "SELECT id FROM users WHERE certificate_hash='" .
			pg_escape_string($certificate_hash) .
			"'" .
			" ORDER BY name,givenname,login";

    $result = $this->db->select($sql);

	if ($result->isError() || $result->num_row() == 0){
		$this->errorMsg = "User::getIdFromCertData - chec du mappage de l'utilisateur depuis les informations du certificat";
		return false;
	}

   	while ($row = $result->get_next_row()){
		$resultat[] = $row['id'];
	}
	return $resultat;

  }

	public function getIdFromLogin($login){
		$sql = "SELECT id FROM users WHERE users.login='" . pg_escape_string($login) . "' AND certificate_hash='" . pg_escape_string($this->certificate) . "'";

		$result = $this->db->select($sql);
		if ($result->num_row() == 0 ){
			return false;
		}
		$row = $result->get_next_row();
		return $row['id'];
	}

  /**
   * \brief Mthode de suppression d'un utilisateur de la base de donnes
   * \param $id integer (optionnel) Numro d'identifiant de l'utilisateur, si non spcifi, entit en cours
   * \return true si succs, false sinon
  */
  public function delete($id = false) {
    // Efface l'entit spcifie par $id ou alors l'entit courante si pas d'id
    if (! $id) {
      if (isset($this->id) && ! empty($this->id)) {
		$id = $this->id;
      } else {
		$this->errorMsg = "Pas d'identifiant pour l'entit a supprimer";
		return false;
      }
    }

	if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

	$sql = "DELETE FROM users_perms WHERE user_id=" . $id;

    if (! $this->db->exec($sql)) {
	  $this->errorMsg = "Erreur lors de la suppression des permissions de l'utilisateur.";
	  $this->db->rollback();
	  return false;
    }

	if (! parent::delete($id)) {
	  $this->db->rollback();
      return false;
	}

	if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

	return true;
  }

  /**
   * \brief Mthode de rcupration de la liste des utilisateurs
   * \param $cond chane (optionnel) : condition  appliquer sur la requte SQL
   * \return Un tableau contenant les donnes des utilisateurs
  */
  public function getUsersList($cond = "") {
	if (! $this->pagerInit('users.id, users.name, users.givenname, users.email, users.role, users.authority_group_id, users.telephone, users.status, users.authority_id, authorities.name AS authority_name, users.login', 'users LEFT OUTER JOIN authorities ON users.authority_id=authorities.id', $cond, 'users.name', null, null, "ASC")) {
	  return false;
	}

    return $this->data;
  }
  
  public function getUserSiren()
  {
  	$sql ="SELECT siren FROM users, authorities WHERE users.authority_id=authorities.id AND users.id=".$this->id;
    $db =DatabasePool::getInstance();

  
    $result = $this->db->select($sql);

    if (! $result->isError() && $result->num_row() == 1) {
      $row = $result->get_next_row();
      return $row["siren"];
    } else {
      $this->errorMsg = "User::getUserSiren - chec du mappage de l'utilisateur depuis les informations du certificat";
      return false;
    }
	}

	public function cloneCertificat($new_id){
		$clone = new User($new_id);
		$clone->init();
		foreach (array('subject_dn', 'issuer_dn', 'certificate', 'cert_not_before', 'cert_not_after', 'cert_serial', 'certificate_hash') as $info) {
			$this->set($info,$clone->get($info));
		}
	}

  /**
   * \brief Mthode permettant de fixer la valeur d'un attribut
   * \param $name chane : Nom de l'attribut
   * \param $val : valeur de l'attribut
  */
  public function set($name, $val) {
	switch ($name) {
	case "role":
	  if (array_search($val, array_keys($this->roleTypes)) === false) {
		$val = 'USER';
	  }
	  break;
	}

	parent::set($name, $val);
  }
	
	public function getAllPossibleAuthority(){
		assert('$this->id');
		if ($this->isSuper()){
			$sql = "SELECT id,name FROM authorities ORDER by name";
		} elseif ($this->isGroupAdmin()){
			$sql = "SELECT id,name FROM authorities WHERE authority_group_id=".$this->get('authority_group_id') . " ORDER by name";
		} else {
			return array($this->get("authority_id") => '');
		}
		$result = array();
		foreach ($this->db->fetchAll($sql) as $r){
			$result[$r['id']] = $r['name'];
		}
		return $result;
	}
	
}

