<?php
/*
 * TDTIS - Copyright 2006 Alternance-Soft
 * Contributeur : Jrme Schell, Aot 2006 
 *
 * contact@alternancesoft.com
 *
 * Ce logiciel est un programme informatique servant  la
 * dmatrialisation de l'administration. 
 *
 * Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilit au code source et des droits de copie,
 * de modification et de redistribution accords par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
 * seule une responsabilit restreinte pse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concdants successifs.
 *
 * A cet gard  l'attention de l'utilisateur est attire sur les risques
 * associs au chargement,   l'utilisation,   la modification et/ou au
 * dveloppement et  la reproduction du logiciel par l'utilisateur tant 
 * donn sa spcificit de logiciel libre, qui peut le rendre complexe  
 * manipuler et qui le rserve donc  des dveloppeurs et des professionnels
 * avertis possdant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invits  charger  et  tester  l'adquation  du
 * logiciel  leurs besoins dans des conditions permettant d'assurer la
 * scurit de leurs systmes et ou de leurs donnes et, plus gnralement, 
 *  l'utiliser et l'exploiter dans les mmes conditions de scurit. 
 *
 * Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accept les
 * termes.
*/
/**
 * \class ActesBatchFile ActesBatchFile.class.php
 * \brief Cette classe permet de grer les fichiers contenus dans les lots de transactions ACTES
 * \author Jrme Schell <j.schell@alternancesoft.com>
 * \date 08.02.2007
 * 
 *
 * Cette classe fournit des mthodes de gestion des fichiers contenus dans les lots de transactions
 * Actes
 *
 * Modifications :
 * Auteur   Date       Commentaire
 *
 */

require_once(SITEROOT . "/class/DataObject.class.php");

class ActesBatchFile extends DataObject {
  protected $objectName = "actes_batch_files";
  
  protected $batch_id;
  protected $transaction_id;
  protected $filename;
  protected $filesize;
  protected $signature;
  protected $status;

  protected $storage_dir;
  protected $tmpFile;
  protected $tmpName;

  protected $dbFields = array(
							  "batch_id" => array( "descr" => "Lot contenant ce fichier", "type" => "isInt", "mandatory" => true),
							  "transaction_id" => array( "descr" => "Transaction issu du fichier", "type" => "isInt", "mandatory" => false),
							  "filename" => array( "descr" => "Nom du fichier sur le serveur", "type" => "isString", "mandatory" => true),
							  "filesize" => array( "descr" => "Taille du fichier", "type" => "isInt", "mandatory" => true),
							  "signature" => array( "descr" => "Signature lectronique du fichier", "type" => "isString", "mandatory" => false),
							  "status" => array( "descr" => "Statut du fichier", "type" => "isString", "mandatory" => true)
							  );

  private $fileStatus = array(
							  "UNPRO" => "Non trait",
							  "PRO" => "Trait"
							  );

  /**
   * \brief Constructeur d'un fichier de lot
   * \param id integer Numro d'identifiant d'un fichier de lot existant avec lequel initialiser l'objet
   */
  public function __construct($id = false) {
	parent::__construct($id);

	if (isset($this->id)) {
	  if (! $this->init()) {
		unset($this->id);
	  }
	}
  }

  /**
   * \brief Dtermination si un fichier est dj trait ou pas
   * \return True si le fichier est trait, false sinon
   */
  public function isProcessed() {
	if (strcmp($this->status, 'PRO') == 0) {
	  return true;
	} else {
	  return false;
	}
  }

  /**
   * \brief Positionnement du statut sur "Non trait"
   */
  public function setUnprocessed() {
	$this->status = 'UNPRO';
  }

  /**
   * \brief Positionnement du statut sur "Trait"
   */
  public function setProcessed() {
	$this->status = 'PRO';
  }

  /**
   * \brief Mthode d'obtention du chemin absolu vers le fichier
   * \return Chemin du fichier sous forme de chane
   */
  public function getAbsoluteFilePath() {
	if (! empty($this->filename)) {
	  return ACTES_BATCHES_UPLOAD_ROOT . "/" . $this->filename;
	} else {
	  return null;
	}
  }

  /**
   * \brief Mthode d'obtention du nom de base du fichier
   * \return Nom de base du fichier
   */
  public function getDisplayName() {
	return basename($this->filename);
  }

  /**
   * \brief Mise en place du fichier dans le rpertoire de stockage du lot
   * \return True en cas de succs, false sinon
   */
  public function putTempFileInStorageDir() {
	if (! move_uploaded_file($this->tmpFile, $this->getAbsoluteFilePath())) {
	  return false;
	}

	return true;
  }

  /**
   * \brief Suppression physique du fichier du lot
   * \return True en cas de succs, false sinon
   */
  public function deleteFile() {
	return Helpers::deleteFromFS(ACTES_BATCHES_UPLOAD_ROOT . "/" . $this->filename);
  }

  /**
   * \brief Mthode d'enregistrement d'un lot dans la base de donnes
   * \param $validate boolen (optionnel) Demande la validation ou non des donnes de l'entit avant enregistrement (true par dfaut)
   * \return true si succs, false sinon
   */
  public function save($validate = true) {
	$new = false;
	if ($this->isNew()) {
	  $new = true;

	  if (empty($this->storage_dir) || empty($this->tmpFile) || empty($this->tmpName)) {
		$this->errorMsg = "Informations manquantes pour la cration du fichier de lot.";
		return false;
	  }

	  $this->filename = $this->storage_dir . "/" . $this->tmpName;

	  if (! $this->putTempFileInStorageDir()) {
		$this->errorMsg = "Erreur lors de la mise en place du fichier du lot.";
		return false;
	  }

	  if (! $this->filesize = @filesize($this->getAbsoluteFilePath())) {
		$this->errorMsg = "Erreur dtermination taille fichier.";
		return false;
	  }
	}

    if (! (parent::save($validate))) {
	  return false;
	}

	return true;
  }

  /**
   * \brief Mthode de suppression d'un fichier de lot
   * \return true si succs, false sinon
  */
  public function delete() {
	if (! $this->deleteFile()) {
	  return false;
	}

	if (! parent::delete()) {
	  return false;
	}

	return true;
  }

  /**
   * \brief Mthode qui renvoie le fichier au navigateur
   * \return True en cas de succs, false sinon
   */
  public function sendFile() {
	$ret_value = false;

	if (isset($this->filename)) {
	  if (file_exists($this->getAbsoluteFilePath())) {
		if (! Helpers::sendFileToBrowser($this->getAbsoluteFilePath(), basename($this->getAbsoluteFilePath()), @mime_content_type($this->getAbsoluteFilePath()))) {
		  $this->errorMsg = "Erreur envoi fichier";
		} else {
		  $ret_value = true;
		}
	  } else {
		$this->errorMsg = "Ce fichier n'est pas/plus disponible.";
	  }
	}

	return $ret_value;
  }


  /**********************/
  /* Mthodes statiques */
  /**********************/

  /**
   * \brief Rcupration des id des fichiers associs  un lot
   * \param id integer Numro d'identifiant du lot
   * \return Un tableau contenant les id des fichiers
   */
  public static function getFilesIdForBatch($id) {
	$tabFiles = array();

	if (is_numeric($id)) {
	  $sql = "SELECT id FROM actes_batch_files WHERE batch_id=" . $id . " ORDER BY id";

	  $db =DatabasePool::getInstance();

	  $result = $db->select($sql);

	  if (! $result->isError()) {
		while ($row = $result->get_next_row()) {
		  $tabFiles[] = $row["id"];
		}
	  }
	}

    return $tabFiles;
  }
}
?>
