<?php
/*
 * TDTIS - Copyright 2006 Alternance-Soft
 * Contributeur : Jrme Schell, Aot 2006 
 *
 * contact@alternancesoft.com
 *
 * Ce logiciel est un programme informatique servant  la
 * dmatrialisation de l'administration. 
 *
 * Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilit au code source et des droits de copie,
 * de modification et de redistribution accords par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
 * seule une responsabilit restreinte pse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concdants successifs.
 *
 * A cet gard  l'attention de l'utilisateur est attire sur les risques
 * associs au chargement,   l'utilisation,   la modification et/ou au
 * dveloppement et  la reproduction du logiciel par l'utilisateur tant 
 * donn sa spcificit de logiciel libre, qui peut le rendre complexe  
 * manipuler et qui le rserve donc  des dveloppeurs et des professionnels
 * avertis possdant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invits  charger  et  tester  l'adquation  du
 * logiciel  leurs besoins dans des conditions permettant d'assurer la
 * scurit de leurs systmes et ou de leurs donnes et, plus gnralement, 
 *  l'utiliser et l'exploiter dans les mmes conditions de scurit. 
 *
 * Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accept les
 * termes.
*/
?>
<?php
/**
 * \class ActesTransmissionWindow ActesTransmissionWindow.class.php
 * \brief Cette classe permet de grer les fentres de transmission ACTES
 * \author Jrme Schell <j.schell@alternancesoft.com>
 * \date 23.08.2006
 * 
 *
 * Cette classe fournit des mthodes de gestion des fentres
 * de transmission ACTES.
 *
 * Modifications :
 * Auteur   Date       Commentaire
 *
 */

require_once(SITEROOT . "/class/DataObject.class.php");

class ActesTransmissionWindow extends DataObject {
  protected $objectName = "actes_transmission_windows";
  
  protected $rate_limit;

  protected $dbFields = array( "rate_limit" => array( "descr" => "Limitation de dbit", "type" => "isInt", "mandatory" => true)
							   );

  protected $window_start_date;
  protected $window_start_stamp;
  protected $window_end_date;
  protected $window_end_stamp;

  private $windowHours;

  /**
   * \brief Constructeur d'une entre de fentre de transmission
   * \param id integer Numro d'identifiant d'une entre existante avec laquelle initialiser l'objet
   */
  public function __construct($id = false) {
	parent::__construct($id);
  }

  /**
   * \brief Mthode initialisant l'entit avec l'identifiant courant
   * \return true si succs, false sinon
  */
  public function init() {
	if (parent::init()) {
	  $sql = "SELECT MIN(window_begin) AS min, MAX(window_end) AS max FROM actes_transmission_window_hours WHERE transmission_window_id = " . $this->id;

	  $result = $this->db->select($sql);

	  if (! $result->isError()) {
		$row = $result->get_next_row();
		$this->window_start_date = $row["min"];
		$this->window_end_date = $row["max"];
	  } else {
		return false;
	  }
	} else {
	  return false;
	}

	return true;
  }

  /**
   * \brief Mthode d'enregistrement d'une entit dans la base de donnes
   * \param $validate boolen (optionnel) Demande la validation ou non des donnes de l'entit avant enregistrement (true par dfaut)
   * \return true si succs, false sinon
   */
  public function save($validate = true) {
	$new = false;
	if ($this->isNew()) {
	  $new = true;
	}

    if (! ($sql = parent::save($validate, true))) {
	  return false;
	}

	if (! $new) {
	  $hours = $this->computeHours($this->getWindowHours());
	} else {
	  $hours = $this->computeHours();
	}

	if ($hours === false) {
	  $this->errorMsg = "Erreur de rcupration des heures de la fentre.";
	  return false;
	} elseif (count($hours) <= 0) {
	  $this->errorMsg = "La fentre a une dure nulle.";
	  return false;
	}

	if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

    if (! $this->db->exec($sql)) {
      $this->errorMsg = "Erreur lors de la sauvegarde de la fentre.";
	  $this->db->rollback();
      return false;
    }
	// Suppression des heures existante
	$sql = "DELETE FROM actes_transmission_window_hours WHERE transmission_window_id=" . $this->id;
    if (! $this->db->exec($sql)) {
      $this->errorMsg = "Erreur lors de la rinitialisation des heures de la fentre.";
	  $this->db->rollback();
      return false;
    }

	// Insertion des heures de la fentre
	foreach ($hours as $hour) {
	  $sql = "INSERT INTO actes_transmission_window_hours (transmission_window_id, window_begin, window_end, consumed) VALUES (" . $this->id . ", '"
		. $hour["window_begin"] . "', '" . $hour["window_end"] . "', " . $hour["consumed"] . ")";

	  if (! $this->db->exec($sql)) {
		$this->errorMsg = "Erreur lors de l'insertion des heures de la fentre.";
		$this->db->rollback();
		return false;
	  }
	}

	if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

    return true;
  }

  /**
   * \brief Mthode de suppression d'une fentre de la base de donnes
   * \param $id integer (optionnel) Numro d'identifiant de la fentre, si non spcifi, entit en cours
   * \return true si succs, false sinon
  */
  public function delete($id = false) {
    // Efface l'entit spcifie par $id ou alors l'entit courante si pas d'id
    if (! $id) {
      if (isset($this->id) && ! empty($this->id)) {
		$id = $this->id;
      } else {
		$this->errorMsg = "Pas d'identifiant pour l'entit a supprimer";
		return false;
      }
    }

	if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

	$sql = "DELETE FROM actes_transmission_window_hours WHERE transmission_window_id=" . $id;

    if (! $this->db->exec($sql)) {
	  $this->errorMsg = "Erreur lors de la suppression des heures associes  la fentre.";
	  $this->db->rollback();
	  return false;
    }

	if (! parent::delete($id)) {
	  $this->db->rollback();
      return false;
	}

	if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

	return true;
  }

  /**
   * \brief Mthode de calcul des heures de la fentre courante
   * \param $currentHours tableau (optionnel) : Tableau contenant les heures actuelles de la fentre en cas de modification
   * \return Un tableau des heures calcules
   */
  private function computeHours($currentHours = false) {
	// Nombre d'heures ncessaires
	$nb_hours = ceil(($this->window_end_stamp - $this->window_start_stamp) / 3600);

	$newHours = array();

	for ($i = 0; $i < $nb_hours; $i++) {
	  $start_stamp = $this->window_start_stamp + $i * 3600;
	  $consumed = 0;

	  if ($currentHours) {
		foreach ($currentHours as $hour) {
		  // Rcupration des octets dj consomms
		  if (Helpers::getTimestampFromBDDDate($hour["window_begin"]) == $start_stamp) {
			if (! empty($hour["consumed"])) {
			  $consumed = $hour["consumed"];
			}
		  }
		}
	  }

	  $start_date = date('Y-m-d H:i:s', $start_stamp);
	  $end_date = date('Y-m-d H:i:s', $start_stamp + 3599);

	  $newHours[] = array("window_begin" => $start_date, "window_end" => $end_date, "consumed" => $consumed);
	}

	return $newHours;
  }

  /**
   * \brief Mthode qui calcule si la fentre courante est en collision avec une fentre existante
   * \return Un tableau d'identifiant des fentres qui interfrent ou false s'il n'y a pas de collision
   */
  public function hasCollision() {
	$hoursId = array();
	if (isset($this->window_start_stamp) && isset($this->window_end_stamp)) {
	  $start_date = date('Y-m-d H:i:s', $this->window_start_stamp);
	  $end_date = date('Y-m-d H:i:s', ($this->window_end_stamp - 1));

	  $sql = "SELECT DISTINCT transmission_window_id FROM actes_transmission_window_hours"
		. " WHERE ((window_begin <= '" . pg_escape_string($start_date) . "' AND window_end >= '" . pg_escape_string($start_date) . "')"
		. " OR (window_begin <= '" . pg_escape_string($end_date) . "' AND window_end >= '" . pg_escape_string($end_date) . "')"
		// Notre intervalle englobe totalement un autre intervalle
		. " OR (window_begin >= '" . pg_escape_string($start_date) . "' AND window_end <= '" . pg_escape_string($end_date) . "'))";

	  // Ne pas tenir compte des chevauchements avec nous-mme
	  if (! $this->isNew()) {
		$sql .= " AND transmission_window_id != " . $this->id;
	  }

	  //echo $sql;
	  //exit();

	  $result = $this->db->select($sql);

	  if (! $result->isError() && $result->num_row() > 0) {
		while ($row = $result->get_next_row()) {
		  $hoursId[] = $row["transmission_window_id"];
		}
	  } else {
		return false;
	  }
	}

	return $hoursId;
  }

  /**
   * \brief Mthode de rcupration des heures correspondant  un fentre
   * \return Un tableau contenant les donnes des heures en cas de succs, false sinon
   */
  private function getWindowHours() {
	if (isset($this->id)) {
	  $sql = "SELECT id, window_begin, window_end, consumed FROM actes_transmission_window_hours WHERE transmission_window_id=" . $this->id;

	  $result = $this->db->select($sql);

	  $windowHours = array();

	  if (! $result->isError()) {
		while ($row = $result->get_next_row()) {
		  $windowHours[] = $row;
		}
	  } else {
		return false;
	  }

	  return $windowHours;
	} else {
	  return false;
	}

	return false;
  }

  /**
   * \brief Mthode de rcupration de la liste des fentres de transmission
   * \param $cond chane (optionnel) : condition  appliquer sur la requte SQL
   * \return Un tableau contenant les donnes des fentres
  */
  public function getWindowsList($cond = "") {
	if (! $this->pagerInit('actes_transmission_windows.id, actes_transmission_windows.rate_limit, MIN(atwh.window_begin) AS start, MAX(atwh.window_end) AS end', 'actes_transmission_windows LEFT OUTER JOIN actes_transmission_window_hours atwh ON actes_transmission_windows.id=atwh.transmission_window_id', $cond . " GROUP BY actes_transmission_windows.id, actes_transmission_windows.rate_limit")) {
	  return false;
	}

    return $this->data;
  }

  /**
   * \brief Mthode de dtermination de l'heure pleine la plus proche de l'heure passe en paramtre
   * \param $date chane : La date au format YYYY-MM-DD
   * \param $hour chane : L'heure au format HH:MM:SS
   * \return Un timestamp correspondant  l'heure pleine
  */
  public static function roundDate($date, $hour) {
	$timestamp = Helpers::ansiDateToTimestamp($date, true);

	$hours = explode(':', $hour);

	if (count($hours) == 3) {
	  $timestamp += $hours[0] * 60 * 60;

	  if ($hours[1] > 30) {
		$timestamp += 3600;
	  }
	}

	return $timestamp;
  }

}